@@ -1,9 +1,50 @@
 import os
 import re
 
-from cola import utils
+from cola import core
 from cola import gitcmds
 from cola import gitcfg
+from cola import utils
+
+
+class Range(object):
+
+    def __init__(self, begin, end):
+        self.begin = self._parse(begin)
+        self.end = self._parse(end)
+
+    def _parse(self, range_str):
+        if ',' in range_str:
+            begin, end = range_str.split(',')
+            return [int(begin), int(end)]
+        else:
+            return [int(range_str), int(range_str)]
+
+    def make(self):
+        return '@@ -%s +%s @@' % (self._span(self.begin), self._span(self.end))
+
+    def set_begin_count(self, count):
+        self._set_count(self.begin, count)
+
+    def set_end_count(self, count):
+        self._set_count(self.end, count)
+
+    def _set_count(self, which, count):
+        if count != which[1]:
+            which[1] = count
+            if count == 1 and which[0] == 0:
+                # the file would be empty in the diff, but we're only
+                # partially applying it, and thus it's not a +0,0 diff
+                # anymore.
+                which[0] = 1
+
+    def _span(self, seq):
+        a = seq[0]
+        b = seq[1]
+        if a == b and a == 1:
+            return '%d' % a
+        else:
+            return '%d,%d' % (a, b)
 
 
 class DiffSource(object):
@@ -17,19 +58,20 @@ class DiffSource(object):
 
 
 class DiffParser(object):
+
     """Handles parsing diff for use by the interactive index editor."""
+
+    HEADER_RE = re.compile(r'^@@ -([0-9,]+) \+([0-9,]+) @@.*')
+
     def __init__(self, model, filename='',
                  cached=True, reverse=False,
                  diff_source=None):
 
-        self._header_re = re.compile('^@@ -(\d+),(\d+) \+(\d+),(\d+) @@.*')
-        self._header_start_re = re.compile('^@@ -(\d+) \+(\d+),(\d+) @@.*')
-        self._headers = []
-
         self._idx = -1
         self._diffs = []
         self._diff_spans = []
         self._diff_offsets = []
+        self._ranges = []
 
         self.config = gitcfg.instance()
         self.head = model.head
@@ -62,12 +104,16 @@ class DiffParser(object):
         if not noop and which < len(self.diff_sel):
             diff = self.diff_sel[which]
             encoding = self.config.file_encoding(self.filename)
-            utils.write(filename, self.header + '\n' + diff + '\n',
-                        encoding=encoding)
+            core.write(filename, self.header + '\n' + diff + '\n',
+                       encoding=encoding)
             return True
         else:
             return False
 
+    def ranges(self):
+        """Return the diff header ranges"""
+        return self._ranges
+
     def diffs(self):
         """Returns the list of diffs."""
         return self._diffs
@@ -77,10 +123,14 @@ class DiffParser(object):
         """
         adds = 0
         deletes = 0
+        existing = 0
         newdiff = []
         local_offset = 0
         offset = self._diff_spans[diff][0]
-        diffguts = '\n'.join(self._diffs[diff])
+
+        ADD = '+'
+        DEL = '-'
+        NOP = ' '
 
         for line in self._diffs[diff]:
             line_start = offset + local_offset
@@ -89,58 +139,44 @@ class DiffParser(object):
             # |line1 |line2 |line3 |
             #   |--selection--|
             #   '-start       '-end
+
             # selection has head of diff (line3)
-            if start < line_start and end > line_start and end < line_end:
-                newdiff.append(line)
-                if line.startswith('+'):
-                    adds += 1
-                if line.startswith('-'):
-                    deletes += 1
+            has_head = start <= line_start and end > line_start and end <= line_end
             # selection has all of diff (line2)
-            elif start <= line_start and end >= line_end:
-                newdiff.append(line)
-                if line.startswith('+'):
-                    adds += 1
-                if line.startswith('-'):
-                    deletes += 1
+            has_all = start <= line_start and end >= line_end
             # selection has tail of diff (line1)
-            elif start >= line_start and start < line_end - 1:
+            has_tail = start >= line_start and start < line_end - 1
+
+            action = line[0:1]
+            if has_head or has_all or has_tail:
                 newdiff.append(line)
-                if line.startswith('+'):
+                if action == ADD:
                     adds += 1
-                if line.startswith('-'):
+                elif action == DEL:
                     deletes += 1
+                elif action == NOP:
+                    existing += 1
             else:
                 # Don't add new lines unless selected
-                if line.startswith('+'):
+                if action == ADD:
                     continue
-                elif line.startswith('-'):
+                elif action == DEL:
                     # Don't remove lines unless selected
                     newdiff.append(' ' + line[1:])
+                    existing += 1
+                elif action == NOP:
+                    newdiff.append(line)
+                    existing += 1
                 else:
                     newdiff.append(line)
 
-        diff_header = self._headers[diff]
-        diff_header_len = len(diff_header)
-
-        if diff_header_len == 4:
-            new_count = diff_header[1] + adds - deletes
-            if new_count != diff_header[3]:
-                header = '@@ -%d,%d +%d,%d @@' % (
-                                diff_header[0],
-                                diff_header[1],
-                                diff_header[2],
-                                new_count)
-                newdiff[0] = header
-        elif diff_header_len == 3:
-            new_count = adds - deletes
-            if new_count != diff_header[2]:
-                header = '@@ -%d +%d,%d @@' % (
-                                diff_header[0],
-                                diff_header[1],
-                                new_count)
-                newdiff[0] = header
+        diff_range = self._ranges[diff]
+        begin_count = existing + deletes
+        end_count = existing + adds
 
+        diff_range.set_begin_count(begin_count)
+        diff_range.set_end_count(end_count)
+        newdiff[0] = diff_range.make()
 
         return (self.header + '\n' + '\n'.join(newdiff) + '\n')
 
@@ -193,46 +229,31 @@ class DiffParser(object):
         """
         total_offset = 0
         self._idx = -1
-        self._headers = []
 
-        for idx, line in enumerate(diff.split('\n')):
-            match = self._header_re.match(line)
-            match_start = None
-            if match is None:
-                match_start = self._header_start_re.match(line)
+        for line in diff.split('\n'):
+            match = self.HEADER_RE.match(line)
             if match:
-                self._headers.append([
-                        int(match.group(1)),
-                        int(match.group(2)),
-                        int(match.group(3)),
-                        int(match.group(4))
-                        ])
+                self._ranges.append(Range(match.group(1), match.group(2)))
+                self._diffs.append([line])
 
-            elif match_start:
-                self._headers.append([
-                        int(match_start.group(1)),
-                        int(match_start.group(2)),
-                        int(match_start.group(3))
-                        ])
-
-            if match or match_start:
-                self._diffs.append( [line] )
                 line_len = len(line) + 1 #\n
                 self._diff_spans.append([total_offset,
-                        total_offset + line_len])
+                                         total_offset + line_len])
                 total_offset += line_len
                 self._diff_offsets.append(total_offset)
                 self._idx += 1
-            else:
-                if self._idx < 0:
-                    errmsg = 'Malformed diff?\n\n%s' % diff
-                    raise AssertionError, errmsg
-                line_len = len(line) + 1
-                total_offset += line_len
+                continue
 
-                self._diffs[self._idx].append(line)
-                self._diff_spans[-1][-1] += line_len
-                self._diff_offsets[self._idx] += line_len
+            if self._idx < 0:
+                errmsg = 'Malformed diff?: %s' % diff
+                raise AssertionError(errmsg)
+
+            line_len = len(line) + 1
+            total_offset += line_len
+
+            self._diffs[self._idx].append(line)
+            self._diff_spans[-1][-1] += line_len
+            self._diff_offsets[self._idx] += line_len
 
     def process_diff_selection(self, selected, offset, selection,
                                apply_to_worktree=False):
@@ -256,6 +277,7 @@ class DiffParser(object):
             selected = False
 
         output = ''
+        error = ''
         status = 0
         # Process diff selection only
         if selected:
@@ -265,14 +287,16 @@ class DiffParser(object):
                 if not contents:
                     continue
                 tmpfile = utils.tmp_filename('selection')
-                utils.write(tmpfile, contents, encoding=encoding)
+                core.write(tmpfile, contents, encoding=encoding)
                 if apply_to_worktree:
-                    stat, out = self.model.apply_diff_to_worktree(tmpfile)
+                    stat, out, err = self.model.apply_diff_to_worktree(tmpfile)
                     output += out
+                    error += err
                     status = max(status, stat)
                 else:
-                    stat, out = self.model.apply_diff(tmpfile)
+                    stat, out, err = self.model.apply_diff(tmpfile)
                     output += out
+                    error += err
                     status = max(status, stat)
                 os.unlink(tmpfile)
         # Process a complete hunk
@@ -282,12 +306,14 @@ class DiffParser(object):
                 if not self.write_diff(tmpfile,idx):
                     continue
                 if apply_to_worktree:
-                    stat, out = self.model.apply_diff_to_worktree(tmpfile)
+                    stat, out, err = self.model.apply_diff_to_worktree(tmpfile)
                     output += out
+                    error += err
                     status = max(status, stat)
                 else:
-                    stat, out = self.model.apply_diff(tmpfile)
+                    stat, out, err = self.model.apply_diff(tmpfile)
                     output += out
+                    error += err
                     status = max(status, stat)
                 os.unlink(tmpfile)
-        return status, output
+        return status, output, error
