@@ -1,5 +1,4 @@
 """Provides commands and queries for Git."""
-import os
 import re
 from cStringIO import StringIO
 
@@ -9,6 +8,7 @@ from cola import utils
 from cola import version
 from cola.compat import set
 from cola.git import git
+from cola.git import STDOUT
 from cola.i18n import N_
 
 config = gitcfg.instance()
@@ -27,15 +27,15 @@ def default_remote(config=None):
 
 def diff_index_filenames(ref):
     """Return a of filenames that have been modified relative to the index"""
-    diff_zstr = git.diff_index(ref, name_only=True, z=True)
-    return _parse_diff_filenames(diff_zstr)
+    out = git.diff_index(ref, name_only=True, z=True)[STDOUT]
+    return _parse_diff_filenames(out)
 
 
 def diff_filenames(*args):
     """Return a list of filenames that have been modified"""
-    diff_zstr = git.diff_tree(name_only=True,
-                              no_commit_id=True, r=True, z=True, *args)
-    return _parse_diff_filenames(diff_zstr)
+    out = git.diff_tree(name_only=True, no_commit_id=True, r=True, z=True,
+                        *args)[STDOUT]
+    return _parse_diff_filenames(out)
 
 
 def diff(args):
@@ -44,22 +44,22 @@ def diff(args):
     :param args: list of arguments to pass to "git diff --name-only"
 
     """
-    diff_zstr = git.diff(name_only=True, z=True, *args)
-    return _parse_diff_filenames(diff_zstr)
+    out = git.diff(name_only=True, z=True, *args)[STDOUT]
+    return _parse_diff_filenames(out)
 
 
-def _parse_diff_filenames(diff_zstr):
-    if diff_zstr:
-        return core.decode(diff_zstr[:-1]).split('\0')
+def _parse_diff_filenames(out):
+    if out:
+        return out[:-1].split('\0')
     else:
         return []
 
 
 def all_files():
     """Return the names of all files in the repository"""
-    ls_files = git.ls_files(z=True)
-    if ls_files:
-        return core.decode(ls_files[:-1]).split('\0')
+    out = git.ls_files(z=True)[STDOUT]
+    if out:
+        return out[:-1].split('\0')
     else:
         return []
 
@@ -76,22 +76,21 @@ def clear_cache():
 
 def current_branch():
     """Return the current branch"""
-    decode = core.decode
     head = git.git_path('HEAD')
     try:
-        key = os.stat(head).st_mtime
+        key = core.stat(head).st_mtime
         if _current_branch.key == key:
             return _current_branch.value
     except OSError:
         pass
-    status, data = git.rev_parse('HEAD', symbolic_full_name=True, with_status=True)
+    status, data, err = git.rev_parse('HEAD', symbolic_full_name=True)
     if status != 0:
         # git init -- read .git/HEAD.  We could do this unconditionally...
         data = _read_git_head(head)
 
     for refs_prefix in ('refs/heads/', 'refs/remotes/', 'refs/tags/'):
         if data.startswith(refs_prefix):
-            value = decode(data[len(refs_prefix):])
+            value = data[len(refs_prefix):]
             _current_branch.key = key
             _current_branch.value = value
             return value
@@ -102,15 +101,15 @@ def current_branch():
 def _read_git_head(head, default='master', git=git):
     """Pure-python .git/HEAD reader"""
     # Legacy .git/HEAD symlinks
-    if os.path.islink(head):
-        refs_heads = os.path.realpath(git.git_path('refs', 'heads'))
-        path = os.path.abspath(head).replace('\\', '/')
+    if core.islink(head):
+        refs_heads = core.realpath(git.git_path('refs', 'heads'))
+        path = core.abspath(head).replace('\\', '/')
         if path.startswith(refs_heads + '/'):
             return path[len(refs_heads)+1:]
 
     # Common .git/HEAD "ref: refs/heads/master" file
-    elif os.path.isfile(head):
-        data = utils.slurp(core.decode(head)).rstrip()
+    elif core.isfile(head):
+        data = core.read(head).rstrip()
         ref_prefix = 'ref: '
         if data.startswith(ref_prefix):
             return data[len(ref_prefix):]
@@ -135,8 +134,8 @@ def branch_list(remote=False):
 
 def for_each_ref_basename(refs, git=git):
     """Return refs starting with 'refs'."""
-    git_output = git.for_each_ref(refs, format='%(refname)')
-    output = core.decode(git_output).splitlines()
+    out = git.for_each_ref(refs, format='%(refname)')[STDOUT]
+    output = out.splitlines()
     non_heads = filter(lambda x: not x.endswith('/HEAD'), output)
     return map(lambda x: x[len(refs) + 1:], non_heads)
 
@@ -150,8 +149,8 @@ def all_refs(split=False, git=git):
     query = (triple('refs/tags', tags),
              triple('refs/heads', local_branches),
              triple('refs/remotes', remote_branches))
-    cmdout = core.decode(git.for_each_ref(format='%(refname)'))
-    for ref in cmdout.splitlines():
+    out = git.for_each_ref(format='%(refname)')[STDOUT]
+    for ref in out.splitlines():
         for prefix, prefix_len, dst in query:
             if ref.startswith(prefix) and not ref.endswith('/HEAD'):
                 dst.append(ref[prefix_len:])
@@ -184,9 +183,9 @@ def tracked_branch(branch=None, config=None):
 
 def untracked_files(git=git):
     """Returns a sorted list of untracked files."""
-    ls_files = git.ls_files(z=True, others=True, exclude_standard=True)
-    if ls_files:
-        return core.decode(ls_files[:-1]).split('\0')
+    out = git.ls_files(z=True, others=True, exclude_standard=True)[STDOUT]
+    if out:
+        return out[:-1].split('\0')
     return []
 
 
@@ -198,9 +197,7 @@ def tag_list():
 
 
 def log(git, *args, **kwargs):
-    return core.decode(git.log(no_color=True,
-                               no_ext_diff=True,
-                               *args, **kwargs))
+    return git.log(no_color=True, no_ext_diff=True, *args, **kwargs)[STDOUT]
 
 
 def commit_diff(sha1, git=git):
@@ -216,23 +213,22 @@ def update_diff_overrides(space_at_eol, space_change,
     _diff_overrides['function_context'] = function_context
 
 
-def _common_diff_opts(config=config):
+def common_diff_opts(config=config):
     submodule = version.check('diff-submodule', version.git_version())
     opts = {
         'patience': True,
         'submodule': submodule,
         'no_color': True,
         'no_ext_diff': True,
-        'with_raw_output': True,
-        'with_stderr': True,
         'unified': config.get('gui.diffcontext', 3),
+        '_raw': True,
     }
     opts.update(_diff_overrides)
     return opts
 
 
 def sha1_diff(git, sha1):
-    return core.decode(git.diff(sha1+'~', sha1, **_common_diff_opts()))
+    return git.diff(sha1+'^!', **common_diff_opts())[STDOUT]
 
 
 def diff_info(sha1, git=git):
@@ -254,7 +250,6 @@ def diff_helper(commit=None,
                 reverse=False,
                 git=git):
     "Invokes git diff on a filepath."
-    encode = core.encode
     if commit:
         ref, endref = commit+'^', commit
     argv = []
@@ -275,15 +270,15 @@ def diff_helper(commit=None,
             argv.append(filename)
             encoding = config.file_encoding(filename)
 
-
     if filename is not None:
-        deleted = cached and not os.path.exists(encode(filename))
+        deleted = cached and not core.exists(filename)
     else:
         deleted = False
 
-    status, diffoutput = git.diff(R=reverse, M=True, cached=cached,
-                                  with_status=True,
-                                  *argv, **_common_diff_opts())
+    status, out, err = git.diff(R=reverse, M=True, cached=cached,
+                                _encoding=encoding,
+                                *argv,
+                                **common_diff_opts())
     if status != 0:
         # git init
         if with_diff_header:
@@ -291,11 +286,11 @@ def diff_helper(commit=None,
         else:
             return ''
 
-    return extract_diff_header(status, deleted, encoding,
-                               with_diff_header, suppress_header, diffoutput)
+    return extract_diff_header(status, deleted,
+                               with_diff_header, suppress_header, out)
 
 
-def extract_diff_header(status, deleted, encoding,
+def extract_diff_header(status, deleted,
                         with_diff_header, suppress_header, diffoutput):
     encode = core.encode
     headers = []
@@ -310,7 +305,7 @@ def extract_diff_header(status, deleted, encoding,
     del_tag = 'deleted file mode '
     output = StringIO()
 
-    diff = core.decode(diffoutput, encoding=encoding).split('\n')
+    diff = diffoutput.split('\n')
     for line in diff:
         if not start and '@@' == line[:2] and '@@' in line[2:]:
             start = True
@@ -318,7 +313,7 @@ def extract_diff_header(status, deleted, encoding,
             output.write(encode(line) + '\n')
         else:
             if with_diff_header:
-                headers.append(encode(line))
+                headers.append(line)
             elif not suppress_header:
                 output.write(encode(line) + '\n')
 
@@ -341,7 +336,8 @@ def format_patchsets(to_export, revs, output='patches'):
 
     """
 
-    outlines = []
+    outs = []
+    errs = []
 
     cur_rev = to_export[0]
     cur_master_idx = revs.index(cur_rev)
@@ -366,42 +362,38 @@ def format_patchsets(to_export, revs, output='patches'):
     # Export each patchsets
     status = 0
     for patchset in patches_to_export:
-        newstatus, out = export_patchset(patchset[0],
+        stat, out, err = export_patchset(patchset[0],
                                          patchset[-1],
                                          output='patches',
                                          n=len(patchset) > 1,
                                          thread=True,
                                          patch_with_stat=True)
-        outlines.append(out)
-        if status == 0:
-            status += newstatus
-    return (status, '\n'.join(outlines))
+        outs.append(out)
+        if err:
+            errs.append(err)
+        status = max(stat, status)
+    return (status, '\n'.join(outs), '\n'.join(errs))
 
 
 def export_patchset(start, end, output='patches', **kwargs):
     """Export patches from start^ to end."""
-    return git.format_patch('-o', output, start + '^..' + end,
-                            with_stderr=True,
-                            with_status=True,
-                            **kwargs)
+    return git.format_patch('-o', output, start + '^..' + end, **kwargs)
 
 
 def unstage_paths(args, head='HEAD'):
-    status, output = git.reset(head, '--', with_status=True,
-                               *set(args))
+    status, out, err = git.reset(head, '--', *set(args))
     if status == 128:
         # handle git init: we have to use 'git rm --cached'
         # detect this condition by checking if the file is still staged
         return untrack_paths(args, head=head)
     else:
-        return (status, output)
+        return (status, out, err)
 
 
 def untrack_paths(args, head='HEAD'):
     if not args:
-        return (-1, N_('Nothing to do'))
-    return git.update_index('--', force_remove=True,
-                            with_status=True, *set(args))
+        return (-1, N_('Nothing to do'), '')
+    return git.update_index('--', force_remove=True, *set(args))
 
 
 def worktree_state(head='HEAD'):
@@ -419,7 +411,7 @@ def worktree_state(head='HEAD'):
            state.get('upstream_changed', []))
 
 
-def worktree_state_dict(head='HEAD', update_index=False):
+def worktree_state_dict(head='HEAD', update_index=False, display_untracked=True):
     """Return a dict of files in various states of being
 
     :rtype: dict, keys are staged, unstaged, untracked, unmerged,
@@ -431,7 +423,7 @@ def worktree_state_dict(head='HEAD', update_index=False):
 
     staged, unmerged, staged_submods = diff_index(head)
     modified, modified_submods = diff_worktree()
-    untracked = untracked_files()
+    untracked = display_untracked and untracked_files() or []
 
     # Remove unmerged paths from the modified list
     unmerged_set = set(unmerged)
@@ -471,22 +463,19 @@ def worktree_state_dict(head='HEAD', update_index=False):
 
 
 def diff_index(head, cached=True):
-    decode = core.decode
     submodules = set()
     staged = []
     unmerged = []
 
-    status, output = git.diff_index(head, '--',
-                                    cached=cached, z=True, with_status=True)
+    status, out, err = git.diff_index(head, '--', cached=cached, z=True)
     if status != 0:
         # handle git init
         return all_files(), unmerged, submodules
 
-    while output:
-        rest, output = output.split('\0', 1)
-        name, output = output.split('\0', 1)
+    while out:
+        rest, out = out.split('\0', 1)
+        name, out = out.split('\0', 1)
         status = rest[-1]
-        name = decode(name)
         if '160000' in rest[1:14]:
             submodules.add(name)
         elif status  in 'DAMT':
@@ -501,19 +490,18 @@ def diff_worktree():
     modified = []
     submodules = set()
 
-    status, output = git.diff_files(z=True, with_status=True)
+    status, out, err = git.diff_files(z=True)
     if status != 0:
         # handle git init
-        ls_files = core.decode(git.ls_files(modified=True, z=True))
-        if ls_files:
-            modified = ls_files[:-1].split('\0')
+        out = git.ls_files(modified=True, z=True)[STDOUT]
+        if out:
+            modified = out[:-1].split('\0')
         return modified, submodules
 
-    while output:
-        rest, output = output.split('\0', 1)
-        name, output = output.split('\0', 1)
+    while out:
+        rest, out = out.split('\0', 1)
+        name, out = out.split('\0', 1)
         status = rest[-1]
-        name = core.decode(name)
         if '160000' in rest[1:14]:
             submodules.add(name)
         elif status in 'DAMT':
@@ -526,8 +514,8 @@ def diff_upstream(head):
     tracked = tracked_branch()
     if not tracked:
         return []
-    merge_base = merge_base_to(head, tracked)
-    return diff_filenames(merge_base, tracked)
+    base = merge_base(head, tracked)
+    return diff_filenames(base, tracked)
 
 
 def _branch_status(branch):
@@ -542,9 +530,9 @@ def _branch_status(branch):
             'upstream_changed': staged}
 
 
-def merge_base_to(head, ref):
+def merge_base(head, ref):
     """Given `ref`, return $(git merge-base ref HEAD)..ref."""
-    return git.merge_base(head, ref)
+    return git.merge_base(head, ref)[STDOUT]
 
 
 def merge_base_parent(branch):
@@ -554,26 +542,11 @@ def merge_base_parent(branch):
     return 'HEAD'
 
 
-def eval_path(path):
-    """handles quoted paths."""
-    if path.startswith('"') and path.endswith('"'):
-        return core.decode(eval(path))
-    else:
-        return core.decode(path)
-
-
-def renamed_files(start, end, git=git):
-    difflines = git.diff('%s..%s' % (start, end), M=True,
-                         **_common_diff_opts()).splitlines()
-    return [eval_path(r[12:].rstrip())
-                for r in difflines if r.startswith('rename from ')]
-
-
 def parse_ls_tree(rev):
     """Return a list of(mode, type, sha1, path) tuples."""
-    lines = git.ls_tree(rev, r=True).splitlines()
     output = []
-    regex = re.compile('^(\d+)\W(\w+)\W(\w+)[ \t]+(.*)$')
+    lines = git.ls_tree(rev, r=True)[STDOUT].splitlines()
+    regex = re.compile(r'^(\d+)\W(\w+)\W(\w+)[ \t]+(.*)$')
     for line in lines:
         match = regex.match(line)
         if match:
@@ -586,12 +559,12 @@ def parse_ls_tree(rev):
 
 
 # A regex for matching the output of git(log|rev-list) --pretty=oneline
-REV_LIST_REGEX = re.compile('^([0-9a-f]{40}) (.*)$')
+REV_LIST_REGEX = re.compile(r'^([0-9a-f]{40}) (.*)$')
 
 def parse_rev_list(raw_revs):
     """Parse `git log --pretty=online` output into (SHA-1, summary) pairs."""
     revs = []
-    for line in map(core.decode, raw_revs.splitlines()):
+    for line in raw_revs.splitlines():
         match = REV_LIST_REGEX.match(line)
         if match:
             rev_id = match.group(1)
@@ -608,7 +581,7 @@ def log_helper(all=False, extra_args=None):
     if extra_args:
         args = extra_args
     output = log(git, pretty='oneline', all=all, *args)
-    for line in map(core.decode, output.splitlines()):
+    for line in output.splitlines():
         match = REV_LIST_REGEX.match(line)
         if match:
             revs.append(match.group(1))
@@ -619,22 +592,23 @@ def log_helper(all=False, extra_args=None):
 def rev_list_range(start, end):
     """Return a (SHA-1, summary) pairs between start and end."""
     revrange = '%s..%s' % (start, end)
-    raw_revs = git.rev_list(revrange, pretty='oneline')
-    return parse_rev_list(raw_revs)
+    out = git.rev_list(revrange, pretty='oneline')[STDOUT]
+    return parse_rev_list(out)
 
 
 def commit_message_path():
     """Return the path to .git/GIT_COLA_MSG"""
     path = git.git_path("GIT_COLA_MSG")
-    if os.path.exists(path):
+    if core.exists(path):
         return path
+    return None
 
 
 def merge_message_path():
     """Return the path to .git/MERGE_MSG or .git/SQUASH_MSG."""
     for basename in ('MERGE_MSG', 'SQUASH_MSG'):
         path = git.git_path(basename)
-        if os.path.exists(path):
+        if core.exists(path):
             return path
     return None
 
@@ -645,18 +619,18 @@ def abort_merge():
     git.read_tree('HEAD', reset=True, u=True, v=True)
     # remove MERGE_HEAD
     merge_head = git.git_path('MERGE_HEAD')
-    if os.path.exists(merge_head):
-        os.unlink(merge_head)
+    if core.exists(merge_head):
+        core.unlink(merge_head)
     # remove MERGE_MESSAGE, etc.
     merge_msg_path = merge_message_path()
     while merge_msg_path:
-        os.unlink(merge_msg_path)
+        core.unlink(merge_msg_path)
         merge_msg_path = merge_message_path()
 
 
 def merge_message(revision):
     """Return a merge message for FETCH_HEAD."""
     fetch_head = git.git_path('FETCH_HEAD')
-    if os.path.exists(fetch_head):
-        return git.fmt_merge_msg('--file', fetch_head)
+    if core.exists(fetch_head):
+        return git.fmt_merge_msg('--file', fetch_head)[STDOUT]
     return "Merge branch '%s'" % revision
