@@ -1,17 +1,21 @@
 """Provides the prefix() function for finding cola resources"""
 import os
 import webbrowser
+from os.path import dirname
 
-_modpath = os.path.abspath(__file__)
-if 'share' in __file__ and 'lib' in __file__:
+from cola import core
+
+
+_modpath = core.abspath(__file__)
+if os.path.join('share', 'git-cola', 'lib') in _modpath:
     # this is the release tree
     # __file__ = '$prefix/share/git-cola/lib/cola/__file__.py'
-    _lib_dir = os.path.dirname(os.path.dirname(_modpath))
-    _prefix = os.path.dirname(os.path.dirname(os.path.dirname(_lib_dir)))
+    _lib_dir = dirname(dirname(_modpath))
+    _prefix = dirname(dirname(dirname(_lib_dir)))
 else:
     # this is the source tree
     # __file__ = '$prefix/cola/__file__.py'
-    _prefix = os.path.dirname(os.path.dirname(_modpath))
+    _prefix = dirname(dirname(_modpath))
 
 
 def prefix(*args):
@@ -29,7 +33,7 @@ def html_docs():
     # index.html only exists after the install-docs target is run,
     # so fallback to git-cola.txt.
     htmldocs = doc('html', 'index.html')
-    if os.path.exists(htmldocs):
+    if core.exists(htmldocs):
         return htmldocs
     return doc('git-cola.txt')
 
@@ -51,3 +55,9 @@ def icon(basename):
 def icon_dir():
     """Return the path to the style dir within the cola install tree."""
     return share('icons')
+
+
+def config_home(*args):
+    config = core.getenv('XDG_CONFIG_HOME',
+                         os.path.join(core.expanduser('~'), '.config'))
+    return os.path.join(config, 'git-cola', *args)
