@@ -7,7 +7,6 @@ from PyQt4.QtCore import SIGNAL
 from cola import qtutils
 from cola import difftool
 from cola import gitcmds
-from cola.git import git
 from cola.i18n import N_
 from cola.qtutils import connect_button
 from cola.widgets import defs
@@ -94,9 +93,8 @@ class CompareBranchesDialog(standard.Dialog):
         self.button_close.setText(N_('Close'))
         self.bottom_grid_layt.addWidget(self.button_close, 1, 3, 1, 1)
 
-        self.diff_files = QtGui.QTreeWidget(self.bottom_widget)
+        self.diff_files = standard.TreeWidget(self.bottom_widget)
         self.diff_files.headerItem().setText(0, N_('File Differences'))
-        self.diff_files.setRootIsDecorated(False)
 
         self.bottom_grid_layt.addWidget(self.diff_files, 0, 0, 1, 4)
         self.main_layt.addWidget(self.splitter)
@@ -199,15 +197,15 @@ class CompareBranchesDialog(standard.Dialog):
             branch = gitcmds.current_branch()
             tracked_branch = gitcmds.tracked_branch()
             if tracked_branch:
-                return git.merge_base(branch, tracked_branch)
+                return gitcmds.merge_base(branch, tracked_branch)
             else:
                 remote_branches = gitcmds.branch_list(remote=True)
                 remote_branch = 'origin/%s' % branch
                 if remote_branch in remote_branches:
-                    return git.merge_base(branch, remote_branch)
+                    return gitcmds.merge_base(branch, remote_branch)
 
                 elif 'origin/master' in remote_branches:
-                    return git.merge_base(branch, 'origin/master')
+                    return gitcmds.merge_base(branch, 'origin/master')
                 else:
                     return 'HEAD'
         else:
