@@ -6,10 +6,12 @@ from PyQt4 import QtGui
 from PyQt4.QtCore import Qt
 from PyQt4.QtCore import SIGNAL
 
-import cola
+from cola.i18n import N_
 from cola import qtutils
 from cola import utils
 from cola.compat import set
+from cola.models import main
+from cola.widgets import defs
 
 
 class CompletionLineEdit(QtGui.QLineEdit):
@@ -17,8 +19,7 @@ class CompletionLineEdit(QtGui.QLineEdit):
     def __init__(self, model, parent=None):
         QtGui.QLineEdit.__init__(self, parent)
 
-        from cola.prefs import diff_font
-        self.setFont(diff_font())
+        self.setFont(qtutils.diff_font())
         # used to hide the completion popup after a drag-select
         self._drag = 0
 
@@ -229,7 +230,7 @@ class HighlightDelegate(QtGui.QStyledItemDelegate):
             html = text.replace(self.highlight_text,
                                 '<strong>%s</strong>' % self.highlight_text)
         else:
-            match = re.match('(.*)(%s)(.*)' % re.escape(self.highlight_text),
+            match = re.match(r'(.*)(%s)(.*)' % re.escape(self.highlight_text),
                              text, re.IGNORECASE)
             if match:
                 start = match.group(1) or ''
@@ -361,7 +362,7 @@ class GitCompletionModel(CompletionModel):
 
     def __init__(self, parent):
         CompletionModel.__init__(self, parent)
-        self.cola_model = model = cola.model()
+        self.main_model = model = main.model()
         msg = model.message_updated
         model.add_observer(msg, self.emit_update)
 
@@ -393,7 +394,7 @@ class GitCompletionModel(CompletionModel):
         return []
 
     def dispose(self):
-        self.cola_model.remove_observer(self.emit_update)
+        self.main_model.remove_observer(self.emit_update)
 
 
 class GitRefCompletionModel(GitCompletionModel):
@@ -403,7 +404,7 @@ class GitRefCompletionModel(GitCompletionModel):
         GitCompletionModel.__init__(self, parent)
 
     def matches(self):
-        model = self.cola_model
+        model = self.main_model
         return model.local_branches + model.remote_branches + model.tags
 
 
@@ -414,7 +415,7 @@ class GitBranchCompletionModel(GitCompletionModel):
         GitCompletionModel.__init__(self, parent)
 
     def matches(self):
-        model = self.cola_model
+        model = self.main_model
         return model.local_branches
 
 
@@ -425,7 +426,7 @@ class GitRemoteBranchCompletionModel(GitCompletionModel):
         GitCompletionModel.__init__(self, parent)
 
     def matches(self):
-        model = self.cola_model
+        model = self.main_model
         return model.remote_branches
 
 
@@ -439,7 +440,7 @@ class GitLogCompletionModel(GitRefCompletionModel):
         (matched_refs, dummy_paths, dummy_dirs) =\
                 GitRefCompletionModel.gather_matches(self, case_sensitive)
 
-        file_list = self.cola_model.everything()
+        file_list = self.main_model.everything()
         files = set(file_list)
         files_and_dirs = utils.add_parents(set(files))
 
@@ -478,3 +479,104 @@ GitLogLineEdit = bind_lineedit(GitLogCompletionModel)
 GitRefLineEdit = bind_lineedit(GitRefCompletionModel)
 GitBranchLineEdit = bind_lineedit(GitBranchCompletionModel)
 GitRemoteBranchLineEdit = bind_lineedit(GitRemoteBranchCompletionModel)
+
+
+class GitDialog(QtGui.QDialog):
+
+    def __init__(self, lineedit, title, button_text, parent):
+        QtGui.QDialog.__init__(self, parent)
+        self.setWindowTitle(title)
+        self.setMinimumWidth(333)
+
+        self.label = QtGui.QLabel()
+        self.label.setText(title)
+
+        self.lineedit = lineedit(self)
+        self.setFocusProxy(self.lineedit)
+
+        self.ok_button = QtGui.QPushButton()
+        self.ok_button.setText(button_text)
+        self.ok_button.setIcon(qtutils.apply_icon())
+
+        self.close_button = QtGui.QPushButton()
+        self.close_button.setText(N_('Close'))
+
+        self.button_layout = QtGui.QHBoxLayout()
+        self.button_layout.setMargin(defs.no_margin)
+        self.button_layout.setSpacing(defs.button_spacing)
+        self.button_layout.addStretch()
+        self.button_layout.addWidget(self.ok_button)
+        self.button_layout.addWidget(self.close_button)
+
+        self.main_layout = QtGui.QVBoxLayout()
+        self.main_layout.setMargin(defs.margin)
+        self.main_layout.setSpacing(defs.spacing)
+
+        self.main_layout.addWidget(self.label)
+        self.main_layout.addWidget(self.lineedit)
+        self.main_layout.addLayout(self.button_layout)
+        self.setLayout(self.main_layout)
+
+        qtutils.connect_button(self.ok_button, self.accept)
+        qtutils.connect_button(self.close_button, self.reject)
+
+        self.connect(self.lineedit, SIGNAL('textChanged(const QString&)'),
+                     self.text_changed)
+
+        self.setWindowModality(Qt.WindowModal)
+        self.ok_button.setEnabled(False)
+
+    def text(self):
+        return unicode(self.lineedit.text())
+
+    def text_changed(self, txt):
+        self.ok_button.setEnabled(bool(self.text()))
+
+    def set_text(self, ref):
+        self.lineedit.setText(ref)
+
+    @classmethod
+    def get(cls, title, button_text, parent, default=None):
+        dlg = cls(title, button_text, parent)
+        if default:
+            dlg.set_text(default)
+
+        dlg.show()
+        dlg.raise_()
+
+        def show_popup():
+            x = dlg.lineedit.x()
+            y = dlg.lineedit.y() + dlg.lineedit.height()
+            point = QtCore.QPoint(x, y)
+            mapped = dlg.mapToGlobal(point)
+            dlg.lineedit.popup().move(mapped.x(), mapped.y())
+            dlg.lineedit.popup().show()
+            dlg.lineedit.refresh()
+
+        QtCore.QTimer().singleShot(0, show_popup)
+
+        if dlg.exec_() == cls.Accepted:
+            return dlg.text()
+        else:
+            return None
+
+
+class GitRefDialog(GitDialog):
+
+    def __init__(self, title, button_text, parent):
+        GitDialog.__init__(self, GitRefLineEdit,
+                           title, button_text, parent)
+
+
+class GitBranchDialog(GitDialog):
+
+    def __init__(self, title, button_text, parent):
+        GitDialog.__init__(self, GitBranchLineEdit,
+                           title, button_text, parent)
+
+
+class GitRemoteBranchDialog(GitDialog):
+
+    def __init__(self, title, button_text, parent):
+        GitDialog.__init__(self, GitRemoteBranchLineEdit,
+                           title, button_text, parent)
