@@ -3,21 +3,20 @@ from PyQt4 import QtCore
 from PyQt4.QtCore import Qt
 from PyQt4.QtCore import SIGNAL
 
-import cola
 from cola import gitcmds
-from cola import qt
 from cola import qtutils
 from cola import utils
+from cola.i18n import N_
 from cola.interaction import Interaction
+from cola.models import main
 from cola.widgets import defs
 from cola.widgets import completion
 from cola.widgets.standard import Dialog
-from cola.main.model import MainModel
 
 
 def create_new_branch(revision=''):
     """Launches a dialog for creating a new branch"""
-    model = MainModel()
+    model = main.MainModel()
     model.update_status()
     view = CreateBranchDialog(model, qtutils.active_window())
     if revision:
@@ -54,27 +53,23 @@ class CreateThread(QtCore.QThread):
 
         if track and '/' in revision:
             remote = revision.split('/', 1)[0]
-            status, out = model.git.fetch(remote,
-                                          with_status=True,
-                                          with_stderr=True)
-            self.emit(SIGNAL('command'), status, out)
-            results.append(('fetch', status, out))
+            status, out, err = model.git.fetch(remote)
+            self.emit(SIGNAL('command'), status, out, err)
+            results.append(('fetch', status, out, err))
 
         if status == 0:
-            status, out = model.create_branch(branch, revision,
-                                              force=reset,
-                                              track=track)
-            self.emit(SIGNAL('command'), status, out)
+            status, out, err = model.create_branch(branch, revision,
+                                                   force=reset,
+                                                   track=track)
+            self.emit(SIGNAL('command'), status, out, err)
 
-        results.append(('branch', status, out))
+        results.append(('branch', status, out, err))
         if status == 0 and checkout:
-            status, out = model.git.checkout(branch,
-                                             with_status=True,
-                                             with_stderr=True)
-            self.emit(SIGNAL('command'), status, out)
-            results.append(('checkout', status, out))
+            status, out, err = model.git.checkout(branch)
+            self.emit(SIGNAL('command'), status, out, err)
+            results.append(('checkout', status, out, err))
 
-        cola.model().update_status()
+        main.model().update_status()
         self.emit(SIGNAL('done'), results)
 
 
@@ -85,7 +80,7 @@ class CreateBranchDialog(Dialog):
         Dialog.__init__(self, parent=parent)
         self.setWindowModality(Qt.WindowModal)
         self.setAttribute(Qt.WA_MacMetalStyle)
-        self.setWindowTitle(self.tr('Create Branch'))
+        self.setWindowTitle(N_('Create Branch'))
 
         self.model = model
         self.opts = CreateOpts(model)
@@ -94,16 +89,16 @@ class CreateBranchDialog(Dialog):
         self.progress = QtGui.QProgressDialog(self)
         self.progress.setRange(0, 0)
         self.progress.setCancelButton(None)
-        self.progress.setWindowTitle(self.tr('Create Branch'))
+        self.progress.setWindowTitle(N_('Create Branch'))
         self.progress.setWindowModality(Qt.WindowModal)
 
         self.branch_name_label = QtGui.QLabel()
-        self.branch_name_label.setText(self.tr('Branch Name'))
+        self.branch_name_label.setText(N_('Branch Name'))
 
         self.branch_name = QtGui.QLineEdit()
 
         self.rev_label = QtGui.QLabel()
-        self.rev_label.setText(self.tr('Starting Revision'))
+        self.rev_label.setText(N_('Starting Revision'))
 
         self.revision = completion.GitRefLineEdit()
         current = gitcmds.current_branch()
@@ -111,51 +106,51 @@ class CreateBranchDialog(Dialog):
             self.revision.setText(current)
 
         self.local_radio = QtGui.QRadioButton()
-        self.local_radio.setText(self.tr('Local Branch'))
+        self.local_radio.setText(N_('Local branch'))
         self.local_radio.setChecked(True)
 
         self.remote_radio = QtGui.QRadioButton()
-        self.remote_radio.setText(self.tr('Tracking Branch'))
+        self.remote_radio.setText(N_('Tracking branch'))
 
         self.tag_radio = QtGui.QRadioButton()
-        self.tag_radio.setText(self.tr('Tag'))
+        self.tag_radio.setText(N_('Tag'))
 
         self.branch_list = QtGui.QListWidget()
 
         self.update_existing_label = QtGui.QLabel()
-        self.update_existing_label.setText(self.tr('Update Existing Branch:'))
+        self.update_existing_label.setText(N_('Update Existing Branch:'))
 
         self.no_update_radio = QtGui.QRadioButton()
-        self.no_update_radio.setText(self.tr('No'))
+        self.no_update_radio.setText(N_('No'))
 
         self.ffwd_only_radio = QtGui.QRadioButton()
-        self.ffwd_only_radio.setText(self.tr('Fast Forward Only'))
+        self.ffwd_only_radio.setText(N_('Fast Forward Only'))
         self.ffwd_only_radio.setChecked(True)
 
         self.reset_radio = QtGui.QRadioButton()
-        self.reset_radio.setText(self.tr('Reset'))
+        self.reset_radio.setText(N_('Reset'))
 
         self.options_bottom_layout = QtGui.QHBoxLayout()
         self.options_checkbox_layout = QtGui.QVBoxLayout()
 
         self.fetch_checkbox = QtGui.QCheckBox()
-        self.fetch_checkbox.setText(self.tr('Fetch Tracking Branch'))
+        self.fetch_checkbox.setText(N_('Fetch Tracking Branch'))
         self.fetch_checkbox.setChecked(True)
         self.options_checkbox_layout.addWidget(self.fetch_checkbox)
 
         self.checkout_checkbox = QtGui.QCheckBox()
-        self.checkout_checkbox.setText(self.tr('Checkout After Creation'))
+        self.checkout_checkbox.setText(N_('Checkout After Creation'))
         self.checkout_checkbox.setChecked(True)
         self.options_checkbox_layout.addWidget(self.checkout_checkbox)
 
         self.options_bottom_layout.addLayout(self.options_checkbox_layout)
         self.options_bottom_layout.addStretch()
 
-        self.create_button = qt.create_button(text='Create Branch',
-                                              icon=qtutils.git_icon())
+        self.create_button = qtutils.create_button(text=N_('Create Branch'),
+                                                   icon=qtutils.git_icon())
         self.create_button.setDefault(True)
 
-        self.close_button = qt.create_button(text='Close')
+        self.close_button = qtutils.create_button(text=N_('Close'))
 
         self.branch_name_layout = QtGui.QHBoxLayout()
         self.branch_name_layout.addWidget(self.branch_name_label)
@@ -174,7 +169,7 @@ class CreateBranchDialog(Dialog):
         self.rev_start_textinput_layout.addWidget(self.revision)
 
         self.rev_start_group = QtGui.QGroupBox()
-        self.rev_start_group.setTitle(self.tr('Starting Revision'))
+        self.rev_start_group.setTitle(N_('Starting Revision'))
 
         self.rev_start_layout = QtGui.QVBoxLayout(self.rev_start_group)
         self.rev_start_layout.setMargin(defs.margin)
@@ -190,7 +185,7 @@ class CreateBranchDialog(Dialog):
         self.options_radio_layout.addWidget(self.reset_radio)
 
         self.option_group = QtGui.QGroupBox()
-        self.option_group.setTitle(self.tr('Options'))
+        self.option_group.setTitle(N_('Options'))
 
         self.options_grp_layout = QtGui.QVBoxLayout(self.option_group)
         self.options_grp_layout.setMargin(defs.margin)
@@ -255,25 +250,25 @@ class CreateBranchDialog(Dialog):
         check_branch = False
 
         if not branch or not revision:
-            qtutils.critical('Missing Data',
-                             'Please provide both a branch '
-                             'name and revision expression.')
+            qtutils.critical(N_('Missing Data'),
+                             N_('Please provide both a branch '
+                                'name and revision expression.'))
             return
         if branch in existing_branches:
             if no_update:
-                msg = self.tr("Branch '%s' already exists.")
-                msg = unicode(msg) % branch
-                qtutils.critical('Branch Exists', msg)
+                msg = N_('Branch "%s" already exists.') % branch
+                qtutils.critical(N_('Branch Exists'), msg)
                 return
             # Whether we should prompt the user for lost commits
             commits = gitcmds.rev_list_range(revision, branch)
             check_branch = bool(commits)
 
         if check_branch:
-            qmsg = self.tr("Resetting '%s' to '%s' will lose commits.")
-            msg = unicode(qmsg) % (branch, revision)
+            msg = (N_('Resetting "%(branch)s" to "%(revision)s" '
+                      'will lose commits.') %
+                   dict(branch=branch, revision=revision))
             if ffwd_only:
-                qtutils.critical('Branch Exists', msg)
+                qtutils.critical(N_('Branch Exists'), msg)
                 return
             lines = [msg]
             for idx, commit in enumerate(commits):
@@ -284,14 +279,15 @@ class CreateBranchDialog(Dialog):
                         +'\t' + subject)
                 if idx >= 5:
                     skip = len(commits) - 5
-                    lines.append('\t(%d skipped)' % skip)
+                    lines.append('\t(%s)' % (N_('%d skipped') % skip))
                     break
-            line = unicode(self.tr('Recovering lost commits may not be easy.'))
+            line = N_('Recovering lost commits may not be easy.')
             lines.append(line)
-            if not qtutils.confirm('Reset Branch?',
+            if not qtutils.confirm(N_('Reset Branch?'),
                                    '\n'.join(lines),
-                                   'Reset "%s" to "%s"?' % (branch, revision),
-                                   'Reset Branch',
+                                   (N_('Reset "%(branch)s" to "%(revision)s"?') %
+                                    dict(branch=branch, revision=revision)),
+                                   N_('Reset Branch'),
                                    default=False,
                                    icon=qtutils.icon('undo.svg')):
                 return
@@ -300,12 +296,12 @@ class CreateBranchDialog(Dialog):
         QtGui.QApplication.setOverrideCursor(Qt.WaitCursor)
 
         # Show a nice progress bar
-        self.progress.setLabelText('Updating...')
+        self.progress.setLabelText(N_('Updating...'))
         self.progress.show()
         self.thread.start()
 
-    def thread_command(self, status, output):
-        Interaction.log_status(status, output)
+    def thread_command(self, status, out, err):
+        Interaction.log_status(status, out, err)
 
     def thread_done(self, results):
         self.setEnabled(True)
@@ -313,18 +309,23 @@ class CreateBranchDialog(Dialog):
         QtGui.QApplication.restoreOverrideCursor()
 
         detail_lines = []
-        for (cmd, status, out) in results:
+        for (cmd, status, out, err) in results:
             if status != 0:
                 Interaction.critical(
-                        'Create Branch Error',
-                        '"git %s" returned exit status "%d"' % (cmd, status))
+                        N_('Error Creating Branch'),
+                        (N_('"%(command)s" returned exit status "%(status)d"') %
+                         dict(command='git '+cmd, status=status)))
                 return
             line = '"git %s" returned exit status %d' % (cmd, status)
             detail_lines.append(line)
-            detail_lines.append(out)
+            if out:
+                detail_lines.append(out)
+            if err:
+                detail_lines.append(err)
             detail_lines.append('')
         details = '\n'.join(detail_lines)
-        qtutils.information('Create Branch', 'Branch created',
+        qtutils.information(N_('Create Branch'),
+                            N_('Branch created'),
                             details=details)
         self.accept()
 
