@@ -3,7 +3,6 @@ from PyQt4 import QtCore
 from PyQt4.QtCore import Qt
 
 from cola import cmds
-from cola import qt
 from cola import qtutils
 from cola.i18n import N_
 from cola.qtutils import connect_button
@@ -14,9 +13,9 @@ from cola.widgets import standard
 from cola.widgets import text
 
 
-def create_tag(revision=''):
+def create_tag(name='', ref='', sign=False):
     """Entry point for external callers."""
-    opts = TagOptions(revision)
+    opts = TagOptions(name, ref, sign)
     view = CreateTag(opts, qtutils.active_window())
     view.show()
     return view
@@ -26,11 +25,14 @@ def create_tag(revision=''):
 class TagOptions(object):
     """Simple data container for the CreateTag dialog."""
 
-    def __init__(self, revision):
-        self.revision = revision or 'HEAD'
+    def __init__(self, name, ref, sign):
+        self.name = name or ''
+        self.ref = ref or 'HEAD'
+        self.sign = sign
 
 
 class CreateTag(standard.Dialog):
+
     def __init__(self, opts, parent):
         standard.Dialog.__init__(self, parent=parent)
         self.setWindowModality(QtCore.Qt.WindowModal)
@@ -52,7 +54,8 @@ class CreateTag(standard.Dialog):
         self.input_form_layt.setWidget(0, QtGui.QFormLayout.LabelRole,
                                        self.tag_name_label)
 
-        self.tag_name = text.HintedLineEdit('vX.Y.Z', self)
+        self.tag_name = text.HintedLineEdit(N_('vX.Y.Z'), self)
+        self.tag_name.set_value(opts.name)
         self.tag_name.setToolTip(N_('Specifies the tag name'))
         self.input_form_layt.setWidget(0, QtGui.QFormLayout.FieldRole,
                                        self.tag_name)
@@ -64,6 +67,7 @@ class CreateTag(standard.Dialog):
                                        self.sign_label)
 
         self.sign_tag = QtGui.QCheckBox(self)
+        self.sign_tag.setChecked(opts.sign)
         self.sign_tag.setToolTip(N_('Whether to sign the tag (git tag -s)'))
         self.input_form_layt.setWidget(1, QtGui.QFormLayout.FieldRole,
                                        self.sign_tag)
@@ -87,7 +91,7 @@ class CreateTag(standard.Dialog):
                                        self.rev_label)
 
         self.revision = completion.GitRefLineEdit()
-        self.revision.setText(self.opts.revision)
+        self.revision.setText(self.opts.ref)
         self.revision.setToolTip(N_('Specifies the SHA-1 to tag'))
         self.input_form_layt.setWidget(3, QtGui.QFormLayout.FieldRole,
                                        self.revision)
@@ -96,12 +100,12 @@ class CreateTag(standard.Dialog):
         self.button_hbox_layt = QtGui.QHBoxLayout()
         self.button_hbox_layt.addStretch()
 
-        self.create_button = qt.create_button(text=N_('Create Tag'),
-                                              icon=qtutils.git_icon())
+        self.create_button = qtutils.create_button(text=N_('Create Tag'),
+                                                   icon=qtutils.git_icon())
         self.button_hbox_layt.addWidget(self.create_button)
         self.main_layt.addLayout(self.button_hbox_layt)
 
-        self.close_button = qt.create_button(text=N_('Close'))
+        self.close_button = qtutils.create_button(text=N_('Close'))
         self.button_hbox_layt.addWidget(self.close_button)
 
         connect_button(self.close_button, self.accept)
