@@ -1,22 +1,39 @@
-import os
-
+from PyQt4 import QtCore
 from PyQt4 import QtGui
 from PyQt4.QtCore import Qt, SIGNAL
 
-import cola
 from cola import cmds
+from cola import core
+from cola import gitcmds
+from cola import gravatar
 from cola import qtutils
 from cola.cmds import run
+from cola.compat import set
 from cola.i18n import N_
-from cola.widgets.text import DiffTextEdit
+from cola.models import main
+from cola.models import selection
+from cola.qtutils import DiffSyntaxHighlighter
+from cola.widgets import defs
+from cola.widgets.text import MonoTextView
+
+
+COMMITS_SELECTED = 'COMMITS_SELECTED'
+
+
+class DiffTextEdit(MonoTextView):
+    def __init__(self, parent, whitespace=True):
+
+        MonoTextView.__init__(self, parent)
+        # Diff/patch syntax highlighter
+        self.highlighter = DiffSyntaxHighlighter(self.document(),
+                                                 whitespace=whitespace)
 
 
 class DiffEditor(DiffTextEdit):
 
     def __init__(self, parent):
         DiffTextEdit.__init__(self, parent)
-        self.model = model = cola.model()
-        self.mode = self.model.mode_none
+        self.model = model = main.model()
 
         self.action_process_section = qtutils.add_action(self,
                 N_('Process Section'),
@@ -58,21 +75,24 @@ class DiffEditor(DiffTextEdit):
                 self.stage_selection)
         self.action_apply_selection.setIcon(qtutils.apply_icon())
 
-        model.add_observer(model.message_mode_about_to_change,
-                           self._mode_about_to_change)
-        model.add_observer(model.message_diff_text_changed, self.setPlainText)
+        model.add_observer(model.message_diff_text_changed, self._emit_text)
 
         self.connect(self, SIGNAL('copyAvailable(bool)'),
                      self.enable_selection_actions)
 
+        self.connect(self, SIGNAL('set_text'), self.setPlainText)
+
+    def _emit_text(self, text):
+        self.emit(SIGNAL('set_text'), text)
+
     # Qt overrides
     def contextMenuEvent(self, event):
         """Create the context menu for the diff display."""
         menu = QtGui.QMenu(self)
-        s = cola.selection()
+        s = selection.selection()
 
         if self.model.stageable():
-            if s.modified and s.modified[0] in cola.model().submodules:
+            if s.modified and s.modified[0] in main.model().submodules:
                 action = menu.addAction(qtutils.icon('add.svg'),
                                         cmds.Stage.name(),
                                         cmds.run(cmds.Stage, s.modified))
@@ -80,7 +100,7 @@ class DiffEditor(DiffTextEdit):
                 menu.addAction(qtutils.git_icon(),
                                N_('Launch git-cola'),
                                cmds.run(cmds.OpenRepo,
-                                    os.path.abspath(s.modified[0])))
+                                        core.abspath(s.modified[0])))
             elif s.modified:
                 action = menu.addAction(qtutils.icon('add.svg'),
                                         N_('Stage Section'),
@@ -94,7 +114,7 @@ class DiffEditor(DiffTextEdit):
                 menu.addAction(self.action_revert_selection)
 
         if self.model.unstageable():
-            if s.staged and s.staged[0] in cola.model().submodules:
+            if s.staged and s.staged[0] in main.model().submodules:
                 action = menu.addAction(qtutils.icon('remove.svg'),
                                         cmds.Unstage.name(),
                                         cmds.do(cmds.Unstage, s.staged))
@@ -102,7 +122,7 @@ class DiffEditor(DiffTextEdit):
                 menu.addAction(qtutils.git_icon(),
                                N_('Launch git-cola'),
                                cmds.do(cmds.OpenRepo,
-                                    os.path.abspath(s.staged[0])))
+                                       core.abspath(s.staged[0])))
             elif s.staged:
                 action = menu.addAction(qtutils.icon('remove.svg'),
                                         N_('Unstage Section'),
@@ -134,15 +154,26 @@ class DiffEditor(DiffTextEdit):
                                       Qt.NoButton,
                                       Qt.NoModifier,
                                       event.orientation())
+
         return DiffTextEdit.wheelEvent(self, event)
 
-    def _mode_about_to_change(self, mode):
-        self.mode = mode
+    def mousePressEvent(self, event):
+        if event.button() == Qt.RightButton:
+            # Intercept right-click to move the cursor to the current position.
+            # setTextCursor() clears the selection so this is only done when
+            # nothing is selected.
+            _, selection_text = self.offset_and_selection()
+            if not selection_text:
+                cursor = self.cursorForPosition(event.pos())
+                self.setTextCursor(cursor)
+
+        return DiffTextEdit.mousePressEvent(self, event)
 
     def setPlainText(self, text):
         """setPlainText(str) while retaining scrollbar positions"""
-        highlight = (self.mode != self.model.mode_none and
-                     self.mode != self.model.mode_untracked)
+        mode = self.model.mode
+        highlight = (mode != self.model.mode_none and
+                     mode != self.model.mode_untracked)
         self.highlighter.set_enabled(highlight)
 
         scrollbar = self.verticalScrollBar()
@@ -154,18 +185,18 @@ class DiffEditor(DiffTextEdit):
         if text is None:
             return
 
-        offset, selection = self.offset_and_selection()
+        offset, selection_text = self.offset_and_selection()
         old_text = unicode(self.toPlainText())
 
         DiffTextEdit.setPlainText(self, text)
 
         # If the old selection exists in the new text then
         # re-select it.
-        if selection and selection in text:
-            idx = text.index(selection)
+        if selection_text and selection_text in text:
+            idx = text.index(selection_text)
             cursor = self.textCursor()
             cursor.setPosition(idx)
-            cursor.setPosition(idx + len(selection),
+            cursor.setPosition(idx + len(selection_text),
                                QtGui.QTextCursor.KeepAnchor)
             self.setTextCursor(cursor)
 
@@ -182,8 +213,8 @@ class DiffEditor(DiffTextEdit):
     def offset_and_selection(self):
         cursor = self.textCursor()
         offset = cursor.position()
-        selection = unicode(cursor.selection().toPlainText())
-        return offset, selection
+        selection_text = unicode(cursor.selection().toPlainText())
+        return offset, selection_text
 
     # Mutators
     def enable_selection_actions(self, enabled):
@@ -193,14 +224,14 @@ class DiffEditor(DiffTextEdit):
         self.action_stage_selection.setEnabled(enabled)
 
     def apply_section(self):
-        s = cola.single_selection()
+        s = selection.single_selection()
         if self.model.stageable() and s.modified:
             self.stage_section()
         elif self.model.unstageable():
             self.unstage_section()
 
     def apply_selection(self):
-        s = cola.single_selection()
+        s = selection.single_selection()
         if self.model.stageable() and s.modified:
             self.stage_selection()
         elif self.model.unstageable():
@@ -252,6 +283,174 @@ class DiffEditor(DiffTextEdit):
                                staged=True, apply_to_worktree=False,
                                reverse=False):
         """Implement un/staging of selected lines or sections."""
-        offset, selection = self.offset_and_selection()
+        if selection.selection_model().is_empty():
+            return
+        offset, selection_text = self.offset_and_selection()
         cmds.do(cmds.ApplyDiffSelection,
-                staged, selected, offset, selection, apply_to_worktree)
+                staged, selected, offset, selection_text, apply_to_worktree)
+
+
+
+class DiffWidget(QtGui.QWidget):
+
+    def __init__(self, notifier, parent):
+        QtGui.QWidget.__init__(self, parent)
+
+        author_font = QtGui.QFont(self.font())
+        author_font.setPointSize(int(author_font.pointSize() * 1.1))
+
+        summary_font = QtGui.QFont(author_font)
+        summary_font.setWeight(QtGui.QFont.Bold)
+
+        policy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,
+                                   QtGui.QSizePolicy.Minimum)
+
+        self.gravatar_label = gravatar.GravatarLabel()
+
+        self.author_label = TextLabel()
+        self.author_label.setTextFormat(Qt.RichText)
+        self.author_label.setFont(author_font)
+        self.author_label.setSizePolicy(policy)
+        self.author_label.setAlignment(Qt.AlignBottom)
+        self.author_label.elide()
+
+        self.summary_label = TextLabel()
+        self.summary_label.setTextFormat(Qt.PlainText)
+        self.summary_label.setFont(summary_font)
+        self.summary_label.setSizePolicy(policy)
+        self.summary_label.setAlignment(Qt.AlignTop)
+        self.summary_label.elide()
+
+        self.sha1_label = TextLabel()
+        self.sha1_label.setTextFormat(Qt.PlainText)
+        self.sha1_label.setSizePolicy(policy)
+        self.sha1_label.setAlignment(Qt.AlignTop)
+        self.sha1_label.elide()
+
+        self.diff = DiffTextEdit(self, whitespace=False)
+        self.tasks = set()
+        self.reflector = QtCore.QObject(self)
+
+        self.info_layout = QtGui.QVBoxLayout()
+        self.info_layout.setMargin(defs.no_margin)
+        self.info_layout.setSpacing(defs.no_spacing)
+        self.info_layout.addWidget(self.author_label)
+        self.info_layout.addWidget(self.summary_label)
+        self.info_layout.addWidget(self.sha1_label)
+
+        self.logo_layout = QtGui.QHBoxLayout()
+        self.logo_layout.setContentsMargins(defs.margin, 0, defs.margin, 0)
+        self.logo_layout.setSpacing(defs.button_spacing)
+        self.logo_layout.addWidget(self.gravatar_label)
+        self.logo_layout.addLayout(self.info_layout)
+
+        self.main_layout = QtGui.QVBoxLayout()
+        self.main_layout.setMargin(defs.no_margin)
+        self.main_layout.setSpacing(defs.spacing)
+        self.main_layout.addLayout(self.logo_layout)
+        self.main_layout.addWidget(self.diff)
+        self.setLayout(self.main_layout)
+
+        notifier.add_observer(COMMITS_SELECTED, self.commits_selected)
+        self.connect(self.reflector, SIGNAL('diff'), self.diff.setText)
+        self.connect(self.reflector, SIGNAL('task_done'), self.task_done)
+
+    def task_done(self, task):
+        try:
+            self.tasks.remove(task)
+        except:
+            pass
+
+    def set_diff_sha1(self, sha1):
+        self.diff.setText('+++ ' + N_('Loading...'))
+        task = DiffInfoTask(sha1, self.reflector)
+        self.tasks.add(task)
+        QtCore.QThreadPool.globalInstance().start(task)
+
+    def commits_selected(self, commits):
+        if len(commits) != 1:
+            return
+        commit = commits[0]
+        sha1 = commit.sha1
+
+        email = commit.email or ''
+        summary = commit.summary or ''
+        author = commit.author or ''
+
+        template_args = {
+                'author': author,
+                'email': email,
+                'summary': summary
+        }
+
+        author_text = ("""%(author)s &lt;"""
+                       """<a href="mailto:%(email)s">"""
+                       """%(email)s</a>&gt;"""
+                       % template_args)
+
+        author_template = '%(author)s <%(email)s>' % template_args
+        self.author_label.set_template(author_text, author_template)
+        self.summary_label.set_text(summary)
+        self.sha1_label.set_text(sha1)
+
+        self.set_diff_sha1(sha1)
+        self.gravatar_label.set_email(email)
+
+
+class TextLabel(QtGui.QLabel):
+    def __init__(self, parent=None):
+        QtGui.QLabel.__init__(self, parent)
+        self.setTextInteractionFlags(Qt.TextSelectableByMouse |
+                                     Qt.LinksAccessibleByMouse)
+        self._display = ''
+        self._template = ''
+        self._text = ''
+        self._elide = False
+        self._metrics = QtGui.QFontMetrics(self.font())
+        self.setOpenExternalLinks(True)
+
+    def elide(self):
+        self._elide = True
+
+    def set_text(self, text):
+        self.set_template(text, text)
+
+    def set_template(self, text, template):
+        self._display = text
+        self._text = text
+        self._template = template
+        self.update_text(self.width())
+        self.setText(self._display)
+
+    def update_text(self, width):
+        self._display = self._text
+        if not self._elide:
+            return
+        text = self._metrics.elidedText(self._template,
+                                        Qt.ElideRight, width-2)
+        if unicode(text) != self._template:
+            self._display = text
+
+    # Qt overrides
+    def setFont(self, font):
+        self._metrics = QtGui.QFontMetrics(font)
+        QtGui.QLabel.setFont(self, font)
+
+    def resizeEvent(self, event):
+        if self._elide:
+            self.update_text(event.size().width())
+            block = self.blockSignals(True)
+            self.setText(self._display)
+            self.blockSignals(block)
+        QtGui.QLabel.resizeEvent(self, event)
+
+
+class DiffInfoTask(QtCore.QRunnable):
+    def __init__(self, sha1, reflector):
+        QtCore.QRunnable.__init__(self)
+        self.sha1 = sha1
+        self.reflector = reflector
+
+    def run(self):
+        diff = gitcmds.diff_info(self.sha1)
+        self.reflector.emit(SIGNAL('diff'), diff)
